<?php
/* Copyright (C) 2011-2012   Stephen Larroque <lrq3000@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *      \file       htdocs/customfields/core/substitutions/functions_customfields.lib.php
 *      \ingroup    customfields
 *      \brief      Substition function for the ODT templating (render accessible all the customfields variable)
 *		\author		Stephen Larroque
 */

/** 		Function called to complete the substitution array (before generating on ODT, or a personalized email)
 * 		functions xxx_completesubstitutionarray are called by make_substitutions() if file
 * 		is inside directory htdocs/core/mymodule/substitutions/
 *
 *		@param	array		$substitutionarray	Array with substitution key=>val
 *		@param	Translate	$langs			Output langs
 *		@param	Object		$object			Object to use to get values
 * 		@return	void					The entry parameter $substitutionarray is modified
 */

// Function to fill the substitution array with all $object fields (so any field passed by a form should be substituted thank's to this function)
// Note: this is NOT necessary for CustomFields, this is only an addon for other devs (since CF is meant to be used by devs, this might ease their burden).
function generic_tag_filling(&$substitutionarray, $object) {
    // Generically add each property of the $object into the substitution array
    foreach ($object as $key=>$value) {
        if (!is_object($value) and !is_resource($value) and !isset($substitutionarray['object_'.$key])) { // only add the property if it is not already defined, and is not an object nor a resource (so it's either a string or a number or an array)
            $substitutionarray['object_'.$key] = $value;
        }
    }
}

// CustomFields substitution function for all ODT templates
// Note: before, $object conveyed any field's data, customfields included, but since Dolibarr v3.2.0 beta, this is not the case anymore, so we need to fetch the customfields datas manually.
// Note2: this function is also automatically called for Dolibarr emails, which is okay but it is also called in the emails config page (?), so then a condition checks that the $object var exists and is not empty, in this case it will just quit
function customfields_completesubstitutionarray(&$substitutionarray,$langs,$object) {
    global $conf,$db;

    if (empty($object)) return; // check that this function is called with an $object for substitution in ODTs (else it may be called in the email config page and create a bug)

    // OPTIONAL : Add generic support for any $object property
    generic_tag_filling($substitutionarray, $object); // must be done before so that we can replace specific values after

    // Adding customfields properties of the $object
    // CustomFields
    if ($conf->global->MAIN_MODULE_CUSTOMFIELDS) { // if the customfields module is activated...
        include_once(DOL_DOCUMENT_ROOT.'/customfields/lib/customfields_aux.lib.php');
        $customfields = customfields_fill_object($object, null, $langs); // fetch all customfields and fill them inside $object->customfields ($customfields will then only contain the returned instance of CustomFields object, with customfields sql structure in database - it is unused here, but just used as an example)
        customfields_fill_object($object, null, $langs, 'raw', null); // fetch all customfields and their RAW value inside $object->customfields->raw (very useful for ODT conditionals substitutions). We already have one $customfields object, so we don't need another one.

        // -- Begin to populate the substitution array with customfields data
        foreach ($object->customfields as $key=>$value) { // One field at a time

            if ($key == 'raw') continue; // if it's the 'raw' subarray, we skip
            // Add this customfield's data to the substitution array (will automatically be replaced inside the ODT, eg: {cf_user} becomes 'John Doe')
            $substitutionarray[$key] = $value; // adding this value to an odt variable (format: {cf_customfield} by default if varprefix = default 'cf_' )
            $substitutionarray[$key.'_raw'] = $object->customfields->raw->$key; // adding the raw value in the _raw variable (format: {cf_customfield_raw})

        }
    }

}
